#!/usr/bin/perl
#
# dbm-client.pl
#
# Copyright 2001 by Anthony Howe.  All rights reserved.
#
# usage: dbm-client.pl [-h host][-p port] dbname secret [key [value]]
#

$myname = 'dbm-client';

$PORT = 9866;

$HOST = '127.0.0.1';

##########################################################################
#	Nothing to be configured below this point.			 #
##########################################################################
$VERSION = '1.0';
$AUTHOR = 'achowe@snert.com';

use Socket;
use Getopt::Std;
use Sys::Hostname;
use AnyDBM_File;

my $usage = "usage: dbm-client.pl [-h host][-p port] dbname secret [key [value]]\n";

die($usage) unless getopts('h:p:');
die($usage) unless 2 <= @ARGV && @ARGV <= 4;

$HOST = $opt_h if defined $opt_h;
$PORT = $opt_p if defined $opt_p;

#########################################################################
#	Support								#
#########################################################################

socket(S, PF_INET, SOCK_STREAM, getprotobyname('tcp')) || die("socket failed: $!");
my $to = sockaddr_in($PORT, inet_aton($HOST));
connect(S, $to) || die("connection to \"$HOST:$PORT\" failed");

my $rin, $rout;
vec($rin, fileno(S), 1) = 1;

sub request {
	my ($cmd, $dbname, $secret, $key, $value) = @_;

	# Build version 0 request.
	my $request = pack("Ca", 0, $cmd);
	$request .= pack("CCCn", length($dbname), length($secret), length($key), length($value));
	$request .= pack("a*a*a*a*", $dbname, $secret, $key, $value);

	# Send request.
	return undef unless send(S, $request, 0, $to);

	# Setup timeout for response read.	
	return undef unless select($rout = $rin, undef, undef, 30);

	my ($vn, $code, $rlen, $response);
	
	# Read the response header.
	return undef unless recv(S, $response, 4, 0);
	
	# Parse header.
	($vn, $code, $rlen) = unpack("CCn", $response);
	
	# Check version number and header length.
	return undef unless $vn == 0 && length($response) == 4;

	# Read remainder of response.
	return undef unless recv(S, $response, $rlen, 0);

	# Check length of request.
	return undef unless length($response) == $rlen;

	# Check status code.
	return undef unless 0x00 <= $code && $code < 0x10;

	# Is that your final answer?
#	return sprintf("0x%02x", $code)." $response";
	return $response;
}

#########################################################################
#	Main.								#
#########################################################################

if (2 < @ARGV) { 
	my $cmd = @ARGV == 4 ? 's' : 'g';
	print request($cmd, @ARGV) . "\n";
	request('q', @ARGV) . "\n";
	close(S);
	exit 0;
}

while (1) {
	print "$myname> ";
	$c = <STDIN>;
	chomp $c;

	next if $c eq '';

	my @cmd = split('\s+', $c, 3);
	my $c = lc(shift @cmd);
	my $response = request($c, @ARGV, @cmd);

	print "$response\n";

	last if $response =~ /\d+ END$/ || $c eq 'q';
}

close(S);
exit 0;

#########################################################################
#									#
#########################################################################
__END__
